"""
(Cosmic) Tendrils is the generic API for (cosmic) FLOWS project and pipeline.
"""
from .version import version as __version__
from .version import version_info as __version_info__
import tendrils.api
import tendrils.utils
